from setuptools import setup, find_packages

setup(
    name="opportunity-core",
    version="0.1.0",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "requests>=2.31.0",
        "python-dotenv>=1.0.0",
        "loguru>=0.7.0",
        "tweepy>=4.14.0",
        "pydantic>=2.12.4",
        "pydantic-settings>=2.12.0",
        "python-telegram-bot>=22.5",
        "six>=1.16.0",
        "urllib3>=1.26.0",  # paapi5_python_sdk requires urllib3 v1.x but works with 2.x
        "certifi>=2023.7.22",
        "python-dateutil>=2.9.0",
    ],
    python_requires=">=3.11",
    description="Core library for Opportunity Radar - Shared utilities and services",
    author="Mustafa Aykon",
)



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.duration_price import DurationPrice  # noqa: F401,E501
from paapi5_python_sdk.models.offer_availability import OfferAvailability  # noqa: F401,E501
from paapi5_python_sdk.models.offer_condition import OfferCondition  # noqa: F401,E501
from paapi5_python_sdk.models.offer_delivery_info import OfferDeliveryInfo  # noqa: F401,E501
from paapi5_python_sdk.models.offer_merchant_info import OfferMerchantInfo  # noqa: F401,E501


class RentalOfferListing(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "availability": "OfferAvailability",
        "base_price": "DurationPrice",
        "condition": "OfferCondition",
        "delivery_info": "OfferDeliveryInfo",
        "id": "str",
        "merchant_info": "OfferMerchantInfo",
    }

    attribute_map = {
        "availability": "Availability",
        "base_price": "BasePrice",
        "condition": "Condition",
        "delivery_info": "DeliveryInfo",
        "id": "Id",
        "merchant_info": "MerchantInfo",
    }

    def __init__(
        self,
        availability=None,
        base_price=None,
        condition=None,
        delivery_info=None,
        id=None,
        merchant_info=None,
    ):  # noqa: E501
        """RentalOfferListing - a model defined in Swagger"""  # noqa: E501

        self._availability = None
        self._base_price = None
        self._condition = None
        self._delivery_info = None
        self._id = None
        self._merchant_info = None
        self.discriminator = None

        if availability is not None:
            self.availability = availability
        if base_price is not None:
            self.base_price = base_price
        if condition is not None:
            self.condition = condition
        if delivery_info is not None:
            self.delivery_info = delivery_info
        if id is not None:
            self.id = id
        if merchant_info is not None:
            self.merchant_info = merchant_info

    @property
    def availability(self):
        """Gets the availability of this RentalOfferListing.  # noqa: E501


        :return: The availability of this RentalOfferListing.  # noqa: E501
        :rtype: OfferAvailability
        """
        return self._availability

    @availability.setter
    def availability(self, availability):
        """Sets the availability of this RentalOfferListing.


        :param availability: The availability of this RentalOfferListing.  # noqa: E501
        :type: OfferAvailability
        """

        self._availability = availability

    @property
    def base_price(self):
        """Gets the base_price of this RentalOfferListing.  # noqa: E501


        :return: The base_price of this RentalOfferListing.  # noqa: E501
        :rtype: DurationPrice
        """
        return self._base_price

    @base_price.setter
    def base_price(self, base_price):
        """Sets the base_price of this RentalOfferListing.


        :param base_price: The base_price of this RentalOfferListing.  # noqa: E501
        :type: DurationPrice
        """

        self._base_price = base_price

    @property
    def condition(self):
        """Gets the condition of this RentalOfferListing.  # noqa: E501


        :return: The condition of this RentalOfferListing.  # noqa: E501
        :rtype: OfferCondition
        """
        return self._condition

    @condition.setter
    def condition(self, condition):
        """Sets the condition of this RentalOfferListing.


        :param condition: The condition of this RentalOfferListing.  # noqa: E501
        :type: OfferCondition
        """

        self._condition = condition

    @property
    def delivery_info(self):
        """Gets the delivery_info of this RentalOfferListing.  # noqa: E501


        :return: The delivery_info of this RentalOfferListing.  # noqa: E501
        :rtype: OfferDeliveryInfo
        """
        return self._delivery_info

    @delivery_info.setter
    def delivery_info(self, delivery_info):
        """Sets the delivery_info of this RentalOfferListing.


        :param delivery_info: The delivery_info of this RentalOfferListing.  # noqa: E501
        :type: OfferDeliveryInfo
        """

        self._delivery_info = delivery_info

    @property
    def id(self):
        """Gets the id of this RentalOfferListing.  # noqa: E501


        :return: The id of this RentalOfferListing.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this RentalOfferListing.


        :param id: The id of this RentalOfferListing.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def merchant_info(self):
        """Gets the merchant_info of this RentalOfferListing.  # noqa: E501


        :return: The merchant_info of this RentalOfferListing.  # noqa: E501
        :rtype: OfferMerchantInfo
        """
        return self._merchant_info

    @merchant_info.setter
    def merchant_info(self, merchant_info):
        """Sets the merchant_info of this RentalOfferListing.


        :param merchant_info: The merchant_info of this RentalOfferListing.  # noqa: E501
        :type: OfferMerchantInfo
        """

        self._merchant_info = merchant_info

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(RentalOfferListing, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RentalOfferListing):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Product discovery service - crawls Amazon to build ASIN pool."""

import logging
import time
from datetime import UTC, datetime
from typing import Any, Callable, Optional

from sqlalchemy.dialects.postgresql import insert
from sqlalchemy.orm import Session

from opportunity_core.models.database import Product
from opportunity_core.utils.api_helper import call_with_backoff
from opportunity_core.utils.authentication import Authentication, AuthenticationConfig
from paapi5_python_sdk.api.default_api import DefaultApi
from paapi5_python_sdk.models import SearchItemsRequest, SearchItemsResource
from paapi5_python_sdk.rest import ApiException

logger = logging.getLogger(__name__)


class ProductDiscoveryService:
    """Discovers and stores ASINs from Amazon PA API."""

    def __init__(
        self,
        api_client: DefaultApi,
        partner_tag: str,
        partner_type: str,
        marketplace: str,
        db_session: Session,
    ):
        """
        Initialize product discovery service.

        Args:
            api_client: Authenticated PA API client
            partner_tag: Amazon partner tag
            partner_type: Partner type (e.g., 'Associates')
            marketplace: Amazon marketplace domain
            db_session: Database session
        """
        self.api_client = api_client
        self.partner_tag = partner_tag
        self.partner_type = partner_type
        self.marketplace = marketplace
        self.db_session = db_session

    def discover_products(
        self,
        keywords: str,
        search_index: str = "All",
        max_pages: int = 10,
        category: Optional[str] = None,
        on_product_found: Optional[Callable[[Product], None]] = None,
    ) -> int:
        """
        Discover products using keyword search and store in database.

        Args:
            keywords: Search keywords
            search_index: Amazon search index (category)
            max_pages: Maximum number of pages to crawl (max 10)
            category: Optional category tag for discovered products
            on_product_found: Optional callback when product is discovered

        Returns:
            Number of new products discovered
        """
        new_products = 0
        updated_products = 0
        items_per_page = 10  # PA API maximum

        logger.info(f"🔍 Starting product discovery: '{keywords}' (max {max_pages} pages)")

        try:
            for page in range(1, min(max_pages, 10) + 1):
                logger.info(f"  Page {page}/{max_pages}...")

                # Minimal resources for discovery - we only need identifiers
                request = SearchItemsRequest(
                    partner_tag=self.partner_tag,
                    partner_type=self.partner_type,
                    marketplace=self.marketplace,
                    keywords=keywords,
                    search_index=search_index,
                    resources=[
                        SearchItemsResource.ITEMINFO_TITLE,
                        SearchItemsResource.IMAGES_PRIMARY_LARGE,
                    ],
                    item_count=items_per_page,
                    item_page=page,
                )

                # Use backoff for API call
                response = call_with_backoff(
                    self.api_client.search_items,
                    request,
                    max_retries=10,  # Increased from 3 to 10
                    initial_delay=5.0,
                    max_delay=120.0,  # Allow waiting up to 2 minutes
                    backoff_factor=2.0,
                )

                # Process results
                if response and hasattr(response, "search_result") and response.search_result:
                    if hasattr(response.search_result, "items") and response.search_result.items:
                        for item in response.search_result.items:
                            product = self._extract_and_store_product(item, keywords, category)
                            if product:
                                if product.first_discovered_at == product.last_seen_at:
                                    new_products += 1
                                else:
                                    updated_products += 1

                                if on_product_found:
                                    on_product_found(product)
                    else:
                        logger.warning(f"  Page {page}: No items in response")
                        break
                else:
                    logger.warning(f"  Page {page}: Empty response")
                    break

                # Rate limiting (User recommendation: ~1.2s for SearchItems)
                if page < max_pages:
                    time.sleep(12.0)  # Increased to 12.0s to be safe

            logger.info(
                f"✅ Discovery complete: {new_products} new, {updated_products} updated (keywords: '{keywords}')"
            )

        except ApiException as e:
            logger.error(f"PA API error during discovery: {e}")
        except Exception as e:
            logger.error(f"Unexpected error during discovery: {e}")

        return new_products

    @staticmethod
    def generate_keyword_variations(
        base_keywords: list[str], add_letters: bool = True, letters: str = "abcdefghijklmnopqrstuvwxyz"
    ) -> list[str]:
        """
        Generate keyword variations to improve discovery coverage.

        Amazon's SearchItems API returns different results for queries like:
        - "temizlik" vs "temizlik a" vs "temizlik b"

        This helps overcome pagination limits and discover more diverse products.

        Args:
            base_keywords: List of base keywords (e.g., ["temizlik", "kozmetik"])
            add_letters: Whether to add letter suffixes (a-z)
            letters: Letters to append (default: a-z)

        Returns:
            Expanded list with variations

        Example:
            >>> generate_keyword_variations(["temizlik"], add_letters=True)
            ["temizlik", "temizlik a", "temizlik b", ..., "temizlik z"]
        """
        variations = []

        for keyword in base_keywords:
            # Always include the base keyword
            variations.append(keyword)

            # Add letter variations if enabled
            if add_letters:
                for letter in letters:
                    variations.append(f"{keyword} {letter}")

        return variations

    def _extract_and_store_product(
        self, item: Any, discovery_keywords: str, category: Optional[str] = None
    ) -> Optional[Product]:
        """
        Extract product data from API response and store/update in database.

        Args:
            item: PA API item response
            discovery_keywords: Keywords used to discover this product
            category: Optional category tag

        Returns:
            Product instance if successful, None otherwise
        """
        try:
            # Extract ASIN
            asin = item.asin if hasattr(item, "asin") else None
            if not asin:
                return None

            # Extract optional metadata
            title = None
            if hasattr(item, "item_info") and item.item_info:
                if hasattr(item.item_info, "title") and item.item_info.title:
                    title = item.item_info.title.display_value

            image_url = None
            if hasattr(item, "images") and item.images:
                if hasattr(item.images, "primary") and item.images.primary:
                    if hasattr(item.images.primary, "large") and item.images.primary.large:
                        image_url = item.images.primary.large.url

            url = None
            if hasattr(item, "detail_page_url"):
                url = item.detail_page_url

            # Upsert product (insert or update if exists)
            now = datetime.now(UTC)

            # PostgreSQL upsert syntax
            stmt = insert(Product).values(
                asin=asin,
                title=title,
                url=url or f"https://{self.marketplace}/dp/{asin}",
                image_url=image_url,
                category=category,
                discovery_keywords=discovery_keywords,
                first_discovered_at=now,
                last_seen_at=now,
                is_active=True,
                check_priority=1,
            )

            # On conflict, update last_seen_at and metadata
            stmt = stmt.on_conflict_do_update(
                index_elements=["asin"],
                set_={
                    "last_seen_at": now,
                    "title": title if title else Product.title,
                    "url": url if url else Product.url,
                    "image_url": image_url if image_url else Product.image_url,
                    "category": category if category else Product.category,
                    "is_active": True,
                },
            )

            self.db_session.execute(stmt)
            self.db_session.commit()

            # Fetch the product to return
            product = self.db_session.query(Product).filter_by(asin=asin).first()
            return product

        except Exception as e:
            logger.error(f"Error storing product: {e}")
            self.db_session.rollback()
            return None

    def discover_from_keyword_list(
        self,
        keywords_list: list[str],
        search_index: str = "All",
        max_pages_per_keyword: int = 10,
        category: Optional[str] = None,
        use_variations: bool = False,
        variation_letters: str = "abcdefghijklmnopqrstuvwxyz",
    ) -> dict[str, int]:
        """
        Discover products from multiple keyword searches.

        Args:
            keywords_list: List of keyword strings
            search_index: Amazon search index
            max_pages_per_keyword: Pages to crawl per keyword
            category: Category tag for discovered products
            use_variations: Whether to generate keyword variations with letters
            variation_letters: Letters to use for variations (default: a-z)

        Returns:
            Dictionary with statistics: {total_new, total_updated, keywords_searched}

        Example:
            # Simple usage (11 keywords)
            discover_from_keyword_list(["temizlik", "kozmetik"])

            # With variations (11 keywords → 11 + 11*26 = 297 keyword variations)
            discover_from_keyword_list(
                ["temizlik", "kozmetik"],
                use_variations=True
            )

            # With limited variations (first 5 letters only)
            discover_from_keyword_list(
                ["temizlik", "kozmetik"],
                use_variations=True,
                variation_letters="abcde"
            )
        """
        # Generate variations if requested
        if use_variations:
            original_count = len(keywords_list)
            keywords_list = self.generate_keyword_variations(keywords_list, add_letters=True, letters=variation_letters)
            logger.info(f"📝 Generated {len(keywords_list)} keyword variations from {original_count} base keywords")

        total_new = 0
        total_updated = 0

        logger.info("=" * 80)
        logger.info(f"🚀 MULTI-KEYWORD DISCOVERY STARTED")
        logger.info(f"   Keywords: {len(keywords_list)}")
        logger.info(f"   Pages per keyword: {max_pages_per_keyword}")
        logger.info(f"   Category: {category or 'None'}")
        logger.info(f"   Variations: {'Enabled' if use_variations else 'Disabled'}")
        logger.info("=" * 80)

        for i, keywords in enumerate(keywords_list, 1):
            logger.info(f"\n>>> Keyword {i}/{len(keywords_list)}: '{keywords}'")

            new = self.discover_products(
                keywords=keywords,
                search_index=search_index,
                max_pages=max_pages_per_keyword,
                category=category,
            )

            total_new += new

            # Rate limiting between keywords
            if i < len(keywords_list):
                time.sleep(20.0)  # Increased to 20.0s between keywords

        logger.info("\n" + "=" * 80)
        logger.info(f"✅ DISCOVERY COMPLETED")
        logger.info(f"   Total new products: {total_new}")
        logger.info(f"   Keywords searched: {len(keywords_list)}")
        logger.info("=" * 80)

        return {
            "total_new": total_new,
            "keywords_searched": len(keywords_list),
        }



import pytest
from datetime import datetime, timedelta, timezone
from opportunity_core.services.deal_detection import DealDetectionEngine, DealType
from opportunity_core.models.database import PriceHistory, Product


def test_all_time_low_rule(db_session, product_factory, price_history_factory, stats_factory):
    """Test that AllTimeLowRule correctly identifies all-time low prices."""
    # Setup
    product = product_factory(asin="B00TEST_ATL")

    # Create history: Old low was 1000, now it's 900 (10% drop, 100 TL savings > 50 TL)
    price_history_factory(product, price=1000.0, captured_at=datetime.now(timezone.utc) - timedelta(days=10))
    current_price = price_history_factory(product, price=900.0)

    # Stats say min was 1000 (before this update), but we simulate the update
    # In reality, stats are updated before detection. So let's set stats to reflect the NEW low.
    stats = stats_factory(
        product,
        min_price_all_time=900.0,  # Current price IS the new low
        avg_price_30d=1000.0,
    )

    engine = DealDetectionEngine(db_session)

    # Debug prints
    print(f"\nProduct: {product.asin}")
    print(f"Stats: min_all_time={stats.min_price_all_time}, avg_30d={stats.avg_price_30d}")

    deals = list(engine.detect_deals(target_asins=[product.asin]))

    assert len(deals) == 1
    assert deals[0].deal_type == DealType.ALL_TIME_LOW
    assert deals[0].current_price == 900.0
    assert deals[0].discount_percentage == 10.0  # (1000-900)/1000


def test_drop_24h_rule(db_session, product_factory, price_history_factory, stats_factory):
    """Test that Drop24HRule detects significant price drops."""
    product = product_factory(asin="B00TEST_DROP")

    # Current price 800, was 1000 yesterday (200 TL drop > 50 TL threshold)
    price_history_factory(product, price=800.0)

    stats_factory(
        product,
        price_change_24h=-200.0,
        price_change_24h_pct=-20.0,
        min_price_all_time=700.0,  # Not an all time low
    )

    engine = DealDetectionEngine(db_session)
    # Configure rule to accept 15% drop
    engine.rules[1].min_drop_percentage = 15.0

    deals = list(engine.detect_deals(target_asins=[product.asin]))

    assert len(deals) == 1
    assert deals[0].deal_type == DealType.DROP_24H
    assert deals[0].discount_percentage == 20.0


def test_cooldown_logic(db_session, product_factory, price_history_factory, stats_factory):
    """Test that cooldown prevents spamming unless price drops further."""
    product = product_factory(asin="B00TEST_COOL")
    # Initial deal: 200 -> 100 (50% discount, 100 TL savings > 50 TL)
    price_history_factory(product, price=100.0)
    stats = stats_factory(product, min_price_all_time=100.0, avg_price_30d=200.0)

    engine = DealDetectionEngine(db_session, cooldown_hours=24)

    # 1. First detection - should be found
    deals1 = list(engine.detect_deals(target_asins=[product.asin]))
    assert len(deals1) == 1

    # Record the alert
    engine.record_alert(deals1[0], platform="telegram")

    # 2. Immediate second check - should be ignored (cooldown)
    deals2 = list(engine.detect_deals(target_asins=[product.asin]))
    assert len(deals2) == 0

    # 3. Price drops further (85.0) - should be found again
    # Drop must be >10% to trigger AllTimeLowRule now (100 -> 85 is 15%)
    price_history_factory(product, price=85.0)
    # Update stats for new price to trigger AllTimeLowRule
    stats.min_price_all_time = 85.0
    db_session.commit()

    deals3 = list(engine.detect_deals(target_asins=[product.asin]))
    assert len(deals3) == 1
    assert deals3[0].current_price == 85.0



# coding: utf-8

# flake8: noqa

from __future__ import absolute_import

"""
  Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  A copy of the License is located at

      http://www.apache.org/licenses/LICENSE-2.0

  or in the "license" file accompanying this file. This file is distributed
  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
  express or implied. See the License for the specific language governing
  permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from paapi5_python_sdk.api_client import ApiClient


class DefaultApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, access_key=None, secret_key=None, host=None, region=None, api_client=None):
        if not host:
            host = "webservices.amazon.com"
        if not region:
            region = "us-east-1"
        if api_client is None:
            api_client = ApiClient(access_key=access_key, secret_key=secret_key, host=host, region=region)
        self.api_client = api_client

    def get_browse_nodes(self, get_browse_nodes_request, **kwargs):  # noqa: E501
        """get_browse_nodes  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_browse_nodes(get_browse_nodes_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GetBrowseNodesRequest get_browse_nodes_request: GetBrowseNodesRequest (required)
        :return: GetBrowseNodesResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_browse_nodes_with_http_info(get_browse_nodes_request, **kwargs)  # noqa: E501
        else:
            (data) = self.get_browse_nodes_with_http_info(get_browse_nodes_request, **kwargs)  # noqa: E501
            return data

    def get_browse_nodes_with_http_info(self, get_browse_nodes_request, **kwargs):  # noqa: E501
        """get_browse_nodes  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_browse_nodes_with_http_info(get_browse_nodes_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GetBrowseNodesRequest get_browse_nodes_request: GetBrowseNodesRequest (required)
        :return: GetBrowseNodesResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["get_browse_nodes_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in six.iteritems(params["kwargs"]):
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s' to method get_browse_nodes" % key)
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'get_browse_nodes_request' is set
        if "get_browse_nodes_request" not in params or params["get_browse_nodes_request"] is None:
            raise ValueError(
                "Missing the required parameter `get_browse_nodes_request` when calling `get_browse_nodes`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "get_browse_nodes_request" in params:
            body_params = params["get_browse_nodes_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json"])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            "/paapi5/getbrowsenodes",
            "POST",
            "GetBrowseNodes",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="GetBrowseNodesResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_items(self, get_items_request, **kwargs):  # noqa: E501
        """get_items  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_items(get_items_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GetItemsRequest get_items_request: GetItemsRequest (required)
        :return: GetItemsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_items_with_http_info(get_items_request, **kwargs)  # noqa: E501
        else:
            (data) = self.get_items_with_http_info(get_items_request, **kwargs)  # noqa: E501
            return data

    def get_items_with_http_info(self, get_items_request, **kwargs):  # noqa: E501
        """get_items  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_items_with_http_info(get_items_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GetItemsRequest get_items_request: GetItemsRequest (required)
        :return: GetItemsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["get_items_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in six.iteritems(params["kwargs"]):
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s' to method get_items" % key)
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'get_items_request' is set
        if "get_items_request" not in params or params["get_items_request"] is None:
            raise ValueError("Missing the required parameter `get_items_request` when calling `get_items`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "get_items_request" in params:
            body_params = params["get_items_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json"])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            "/paapi5/getitems",
            "POST",
            "GetItems",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="GetItemsResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_variations(self, get_variations_request, **kwargs):  # noqa: E501
        """get_variations  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_variations(get_variations_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GetVariationsRequest get_variations_request: GetVariationsRequest (required)
        :return: GetVariationsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_variations_with_http_info(get_variations_request, **kwargs)  # noqa: E501
        else:
            (data) = self.get_variations_with_http_info(get_variations_request, **kwargs)  # noqa: E501
            return data

    def get_variations_with_http_info(self, get_variations_request, **kwargs):  # noqa: E501
        """get_variations  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_variations_with_http_info(get_variations_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param GetVariationsRequest get_variations_request: GetVariationsRequest (required)
        :return: GetVariationsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["get_variations_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in six.iteritems(params["kwargs"]):
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s' to method get_variations" % key)
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'get_variations_request' is set
        if "get_variations_request" not in params or params["get_variations_request"] is None:
            raise ValueError("Missing the required parameter `get_variations_request` when calling `get_variations`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "get_variations_request" in params:
            body_params = params["get_variations_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json"])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            "/paapi5/getvariations",
            "POST",
            "GetVariations",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="GetVariationsResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def search_items(self, search_items_request, **kwargs):  # noqa: E501
        """search_items  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.search_items(search_items_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param SearchItemsRequest search_items_request: SearchItemsRequest (required)
        :return: SearchItemsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.search_items_with_http_info(search_items_request, **kwargs)  # noqa: E501
        else:
            (data) = self.search_items_with_http_info(search_items_request, **kwargs)  # noqa: E501
            return data

    def search_items_with_http_info(self, search_items_request, **kwargs):  # noqa: E501
        """search_items  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.search_items_with_http_info(search_items_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param SearchItemsRequest search_items_request: SearchItemsRequest (required)
        :return: SearchItemsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["search_items_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in six.iteritems(params["kwargs"]):
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s' to method search_items" % key)
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'search_items_request' is set
        if "search_items_request" not in params or params["search_items_request"] is None:
            raise ValueError("Missing the required parameter `search_items_request` when calling `search_items`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "search_items_request" in params:
            body_params = params["search_items_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/json"])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            "/paapi5/searchitems",
            "POST",
            "SearchItems",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="SearchItemsResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



# coding: utf-8

# flake8: noqa

from __future__ import absolute_import

"""
  Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  A copy of the License is located at

      http://www.apache.org/licenses/LICENSE-2.0

  or in the "license" file accompanying this file. This file is distributed
  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
  express or implied. See the License for the specific language governing
  permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


# import auth into sdk package
from paapi5_python_sdk.auth.sign_helper import AWSV4Auth


# import apis into sdk package
from paapi5_python_sdk.api.default_api import DefaultApi

# import ApiClient
from paapi5_python_sdk.api_client import ApiClient
from paapi5_python_sdk.configuration import Configuration

# import models into sdk package
from paapi5_python_sdk.models.availability import Availability
from paapi5_python_sdk.models.big_decimal import BigDecimal
from paapi5_python_sdk.models.browse_node import BrowseNode
from paapi5_python_sdk.models.browse_node_ancestor import BrowseNodeAncestor
from paapi5_python_sdk.models.browse_node_child import BrowseNodeChild
from paapi5_python_sdk.models.browse_node_info import BrowseNodeInfo
from paapi5_python_sdk.models.browse_nodes_result import BrowseNodesResult
from paapi5_python_sdk.models.by_line_info import ByLineInfo
from paapi5_python_sdk.models.classifications import Classifications
from paapi5_python_sdk.models.condition import Condition
from paapi5_python_sdk.models.content_info import ContentInfo
from paapi5_python_sdk.models.content_rating import ContentRating
from paapi5_python_sdk.models.contributor import Contributor
from paapi5_python_sdk.models.customer_reviews import CustomerReviews
from paapi5_python_sdk.models.deal_details import DealDetails
from paapi5_python_sdk.models.delivery_flag import DeliveryFlag
from paapi5_python_sdk.models.dimension_based_attribute import DimensionBasedAttribute
from paapi5_python_sdk.models.duration_price import DurationPrice
from paapi5_python_sdk.models.error_data import ErrorData
from paapi5_python_sdk.models.external_ids import ExternalIds
from paapi5_python_sdk.models.get_browse_nodes_request import GetBrowseNodesRequest
from paapi5_python_sdk.models.get_browse_nodes_resource import GetBrowseNodesResource
from paapi5_python_sdk.models.get_browse_nodes_response import GetBrowseNodesResponse
from paapi5_python_sdk.models.get_items_request import GetItemsRequest
from paapi5_python_sdk.models.get_items_resource import GetItemsResource
from paapi5_python_sdk.models.get_items_response import GetItemsResponse
from paapi5_python_sdk.models.get_variations_request import GetVariationsRequest
from paapi5_python_sdk.models.get_variations_resource import GetVariationsResource
from paapi5_python_sdk.models.get_variations_response import GetVariationsResponse
from paapi5_python_sdk.models.image_size import ImageSize
from paapi5_python_sdk.models.image_type import ImageType
from paapi5_python_sdk.models.images import Images
from paapi5_python_sdk.models.item import Item
from paapi5_python_sdk.models.item_id_type import ItemIdType
from paapi5_python_sdk.models.item_info import ItemInfo
from paapi5_python_sdk.models.items_result import ItemsResult
from paapi5_python_sdk.models.language_type import LanguageType
from paapi5_python_sdk.models.languages import Languages
from paapi5_python_sdk.models.manufacture_info import ManufactureInfo
from paapi5_python_sdk.models.max_price import MaxPrice
from paapi5_python_sdk.models.merchant import Merchant
from paapi5_python_sdk.models.min_price import MinPrice
from paapi5_python_sdk.models.min_reviews_rating import MinReviewsRating
from paapi5_python_sdk.models.min_saving_percent import MinSavingPercent
from paapi5_python_sdk.models.money import Money
from paapi5_python_sdk.models.multi_valued_attribute import MultiValuedAttribute
from paapi5_python_sdk.models.offer_availability import OfferAvailability
from paapi5_python_sdk.models.offer_availability_v2 import OfferAvailabilityV2
from paapi5_python_sdk.models.offer_condition import OfferCondition
from paapi5_python_sdk.models.offer_condition_note import OfferConditionNote
from paapi5_python_sdk.models.offer_condition_v2 import OfferConditionV2
from paapi5_python_sdk.models.offer_count import OfferCount
from paapi5_python_sdk.models.offer_delivery_info import OfferDeliveryInfo
from paapi5_python_sdk.models.offer_listing import OfferListing
from paapi5_python_sdk.models.offer_listing_v2 import OfferListingV2
from paapi5_python_sdk.models.offer_listings import OfferListings
from paapi5_python_sdk.models.offer_listings_v2 import OfferListingsV2
from paapi5_python_sdk.models.offer_loyalty_points import OfferLoyaltyPoints
from paapi5_python_sdk.models.offer_loyalty_points_v2 import OfferLoyaltyPointsV2
from paapi5_python_sdk.models.offer_merchant_info import OfferMerchantInfo
from paapi5_python_sdk.models.offer_merchant_info_v2 import OfferMerchantInfoV2
from paapi5_python_sdk.models.offer_price import OfferPrice
from paapi5_python_sdk.models.offer_price_v2 import OfferPriceV2
from paapi5_python_sdk.models.offer_program_eligibility import OfferProgramEligibility
from paapi5_python_sdk.models.offer_promotion import OfferPromotion
from paapi5_python_sdk.models.offer_saving_basis import OfferSavingBasis
from paapi5_python_sdk.models.offer_savings import OfferSavings
from paapi5_python_sdk.models.offer_savings_v2 import OfferSavingsV2
from paapi5_python_sdk.models.offer_shipping_charge import OfferShippingCharge
from paapi5_python_sdk.models.offer_sub_condition import OfferSubCondition
from paapi5_python_sdk.models.offer_summary import OfferSummary
from paapi5_python_sdk.models.offer_type import OfferType
from paapi5_python_sdk.models.offers import Offers
from paapi5_python_sdk.models.offers_v2 import OffersV2
from paapi5_python_sdk.models.partner_type import PartnerType
from paapi5_python_sdk.models.price import Price
from paapi5_python_sdk.models.price_type import PriceType
from paapi5_python_sdk.models.product_advertising_api_client_exception import (
    ProductAdvertisingAPIClientException,
)
from paapi5_python_sdk.models.product_advertising_api_service_exception import (
    ProductAdvertisingAPIServiceException,
)
from paapi5_python_sdk.models.product_info import ProductInfo
from paapi5_python_sdk.models.properties import Properties
from paapi5_python_sdk.models.rating import Rating
from paapi5_python_sdk.models.refinement import Refinement
from paapi5_python_sdk.models.refinement_bin import RefinementBin
from paapi5_python_sdk.models.rental_offer_listing import RentalOfferListing
from paapi5_python_sdk.models.rental_offers import RentalOffers
from paapi5_python_sdk.models.saving_basis_type import SavingBasisType
from paapi5_python_sdk.models.search_index import SearchIndex
from paapi5_python_sdk.models.search_items_request import SearchItemsRequest
from paapi5_python_sdk.models.search_items_resource import SearchItemsResource
from paapi5_python_sdk.models.search_items_response import SearchItemsResponse
from paapi5_python_sdk.models.search_refinements import SearchRefinements
from paapi5_python_sdk.models.search_result import SearchResult
from paapi5_python_sdk.models.single_boolean_valued_attribute import (
    SingleBooleanValuedAttribute,
)
from paapi5_python_sdk.models.single_integer_valued_attribute import (
    SingleIntegerValuedAttribute,
)
from paapi5_python_sdk.models.single_string_valued_attribute import (
    SingleStringValuedAttribute,
)
from paapi5_python_sdk.models.sort_by import SortBy
from paapi5_python_sdk.models.technical_info import TechnicalInfo
from paapi5_python_sdk.models.trade_in_info import TradeInInfo
from paapi5_python_sdk.models.trade_in_price import TradeInPrice
from paapi5_python_sdk.models.unit_based_attribute import UnitBasedAttribute
from paapi5_python_sdk.models.variation_attribute import VariationAttribute
from paapi5_python_sdk.models.variation_dimension import VariationDimension
from paapi5_python_sdk.models.variation_summary import VariationSummary
from paapi5_python_sdk.models.variations_result import VariationsResult
from paapi5_python_sdk.models.website_sales_rank import WebsiteSalesRank



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.unit_based_attribute import UnitBasedAttribute  # noqa: F401,E501


class DimensionBasedAttribute(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "height": "UnitBasedAttribute",
        "length": "UnitBasedAttribute",
        "weight": "UnitBasedAttribute",
        "width": "UnitBasedAttribute",
    }

    attribute_map = {
        "height": "Height",
        "length": "Length",
        "weight": "Weight",
        "width": "Width",
    }

    def __init__(self, height=None, length=None, weight=None, width=None):  # noqa: E501
        """DimensionBasedAttribute - a model defined in Swagger"""  # noqa: E501

        self._height = None
        self._length = None
        self._weight = None
        self._width = None
        self.discriminator = None

        if height is not None:
            self.height = height
        if length is not None:
            self.length = length
        if weight is not None:
            self.weight = weight
        if width is not None:
            self.width = width

    @property
    def height(self):
        """Gets the height of this DimensionBasedAttribute.  # noqa: E501


        :return: The height of this DimensionBasedAttribute.  # noqa: E501
        :rtype: UnitBasedAttribute
        """
        return self._height

    @height.setter
    def height(self, height):
        """Sets the height of this DimensionBasedAttribute.


        :param height: The height of this DimensionBasedAttribute.  # noqa: E501
        :type: UnitBasedAttribute
        """

        self._height = height

    @property
    def length(self):
        """Gets the length of this DimensionBasedAttribute.  # noqa: E501


        :return: The length of this DimensionBasedAttribute.  # noqa: E501
        :rtype: UnitBasedAttribute
        """
        return self._length

    @length.setter
    def length(self, length):
        """Sets the length of this DimensionBasedAttribute.


        :param length: The length of this DimensionBasedAttribute.  # noqa: E501
        :type: UnitBasedAttribute
        """

        self._length = length

    @property
    def weight(self):
        """Gets the weight of this DimensionBasedAttribute.  # noqa: E501


        :return: The weight of this DimensionBasedAttribute.  # noqa: E501
        :rtype: UnitBasedAttribute
        """
        return self._weight

    @weight.setter
    def weight(self, weight):
        """Sets the weight of this DimensionBasedAttribute.


        :param weight: The weight of this DimensionBasedAttribute.  # noqa: E501
        :type: UnitBasedAttribute
        """

        self._weight = weight

    @property
    def width(self):
        """Gets the width of this DimensionBasedAttribute.  # noqa: E501


        :return: The width of this DimensionBasedAttribute.  # noqa: E501
        :rtype: UnitBasedAttribute
        """
        return self._width

    @width.setter
    def width(self, width):
        """Sets the width of this DimensionBasedAttribute.


        :param width: The width of this DimensionBasedAttribute.  # noqa: E501
        :type: UnitBasedAttribute
        """

        self._width = width

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(DimensionBasedAttribute, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DimensionBasedAttribute):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License").
You may not use this file except in compliance with the License.
A copy of the License is located at

    http://www.apache.org/licenses/LICENSE-2.0

or in the "license" file accompanying this file. This file is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
"""

"""
    ProductAdvertisingAPI

    https://webservices.amazon.com/paapi5/documentation/index.html  # noqa: E501
"""


import pprint
import re  # noqa: F401

import six

from paapi5_python_sdk.models.browse_node import BrowseNode  # noqa: F401,E501
from paapi5_python_sdk.models.website_sales_rank import WebsiteSalesRank  # noqa: F401,E501


class BrowseNodeInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "browse_nodes": "list[BrowseNode]",
        "website_sales_rank": "WebsiteSalesRank",
    }

    attribute_map = {
        "browse_nodes": "BrowseNodes",
        "website_sales_rank": "WebsiteSalesRank",
    }

    def __init__(self, browse_nodes=None, website_sales_rank=None):  # noqa: E501
        """BrowseNodeInfo - a model defined in Swagger"""  # noqa: E501

        self._browse_nodes = None
        self._website_sales_rank = None
        self.discriminator = None

        if browse_nodes is not None:
            self.browse_nodes = browse_nodes
        if website_sales_rank is not None:
            self.website_sales_rank = website_sales_rank

    @property
    def browse_nodes(self):
        """Gets the browse_nodes of this BrowseNodeInfo.  # noqa: E501


        :return: The browse_nodes of this BrowseNodeInfo.  # noqa: E501
        :rtype: list[BrowseNode]
        """
        return self._browse_nodes

    @browse_nodes.setter
    def browse_nodes(self, browse_nodes):
        """Sets the browse_nodes of this BrowseNodeInfo.


        :param browse_nodes: The browse_nodes of this BrowseNodeInfo.  # noqa: E501
        :type: list[BrowseNode]
        """

        self._browse_nodes = browse_nodes

    @property
    def website_sales_rank(self):
        """Gets the website_sales_rank of this BrowseNodeInfo.  # noqa: E501


        :return: The website_sales_rank of this BrowseNodeInfo.  # noqa: E501
        :rtype: WebsiteSalesRank
        """
        return self._website_sales_rank

    @website_sales_rank.setter
    def website_sales_rank(self, website_sales_rank):
        """Sets the website_sales_rank of this BrowseNodeInfo.


        :param website_sales_rank: The website_sales_rank of this BrowseNodeInfo.  # noqa: E501
        :type: WebsiteSalesRank
        """

        self._website_sales_rank = website_sales_rank

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict()) if hasattr(item[1], "to_dict") else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(BrowseNodeInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BrowseNodeInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
